<?php
namespace app\home\controller;

use app\common\controller\Base;

class Upload extends Base
{
    private $result = [
        'msg'  => '上传成功',
        'code' => 1,
        'data' => [],
    ];
    
    protected function initialize()
    {
        parent::initialize(); 
        
        $this->auth['check'] = true;
        
        $this->auth['cancel_limit'] = 'home/upload/index';
        
        $this->authCheck();
    }
    
    public function index()
    {
        return json($this->result);
    }
    
    public function image()
    {
        // 获取表单上传文件 例如上传了001.jpg
        $file = $this->request->file('file');
        // 移动到框架应用根目录/uploads/ 目录下
        $info = $file->rule('md5')->validate(['size'=>config('upload.size')*1024*1024,'ext'=>config('upload.ext')])->move(config('upload.path'));
        if($info){
            // 输出 jpg
            $this->result['data']['ext'] = $info->getExtension();
            // 输出 md5值
            $this->result['data']['md5'] = $info->hash('md5');
            // 输出 42a79759f284b767dfcb2a0197904287.jpg
            $this->result['data']['fileName'] = $info->getFilename();
            // 输出 20160820/42a79759f284b767dfcb2a0197904287.jpg
            $this->result['data']['saveName'] = str_replace('\\','/',$info->getSaveName());
            // 输出 http://xxxxx/uploads/savename.jpg
            $this->result['data']['cdnUrl'] = $this->thumbCdn($this->result['data']['saveName']);
        }else{
            $this->result['msg']  = $file->getError();
            $this->result['code'] = -1;
            $this->result['data']['cdnUrl'] = $this->site['pathStatic'].'/images/default.png';
        }
        //返回信息
        return json($this->result);
    }

    //生成缩略图
    private function thumbCdn($saveName='')
    {
        //宽度
        $width = input(0);
        //高度
        $height = input(1);
        //填充类型(1,2,3,4,5,6)
        $type = input(2);
        //缩略图名称
        $name = input('name/s','thumb');
        //图片处理并保存
        if($width && $height && $type){
            //加载已上传的图片
            $image = \think\Image::open(config('upload.path').$saveName);
            //缩略图保存名
            $thumbSaveName = xfThumb($saveName,$name);
            //同级目录生成缩略图
            $image->thumb($width,$height,$type)->save(config('upload.path').$thumbSaveName);
            //返回图片路径
            return config('upload.cdn').$thumbSaveName;
        }
        //CDN前缀
        return config('upload.cdn').$saveName;
    }
}
